/******************************************************************************
*
* project name:  TICT-Explorer
* file name:     protect.c
* initial date:  16/01/2001
* author:        thomas.nussbaumer@gmx.net
*
* description:   anticrash handler ported from DoorOS 0.98 source code
*
* $Id: protect.c,v 1.3 2002/02/07 18:01:18 tnussb Exp $
*
******************************************************************************/

#define EXCEPTION_NONE  (-1)

volatile short exception_occured = EXCEPTION_NONE;

#define RESET_EXCEPTION()     {exception_occured = EXCEPTION_NONE;}
#define EXCEPTION_TRIGGERED() (exception_occured != -1)
#define GET_EXCEPTION_CODE()  (exception_occured)


extern void exceptionhandlers(void);
asm ("exceptionhandlers:\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #0,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #1,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #2,%d0\n"
"    move.w  #0x2700,%sr\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #3,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #4,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #5,%d0\n"
"    move.w  #0x2700,%sr\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #6,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #7,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #8,%d0\n"
"    move.w  #0x2700,%sr\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #9,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #10,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #11,%d0\n"
"    move.w  #0x2700,%sr\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #12,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #13,%d0\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #14,%d0\n"
"    move.w  #0x2700,%sr\n"
"    bra.w   crash\n"
"    move.w  #0x2700,%sr\n"
"    move.w  #15,%d0\n"
"crash:\n"
"    move.w  %d0,exception_occured\n"
"    move.w  #0x700,%sr\n"
"    dc.w    0xa006");    // trigger line1010 exception with value 0x6 -> throws exception!

unsigned char vec_offsets[16] = {
    0xC,            // ADDRESS ERROR
    0x10,           // ILLEGAL INSTRUCTION
    0x14,           // DIV BY ZERO
    0x18,           // CHK
    0x1C,           // TRAPV
    0x20,           // PRIV VIOLATION
    0x24,           // TRACE
    0x2C,           // LINE 1111
    0x60,           // SPURIOUS
    0x7C,           // MEMORY VIOLATION
    0x80+4*5,       // TRAP 5
    0x80+4*6,       // TRAP 6
    0x80+4*7,       // TRAP 7
    0x80+4*13,      // TRAP 13
    0x80+4*14,      // TRAP 14
    0x80+4*15};     // TRAP 15 : ER_Throw


typedef unsigned long PROTECTION_T[23];

//PROTECTION_T vec_backups = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23};

/*
extern void DisableWriteProtection(void);
asm ("DisableWriteProtection:
     movem.l %a0/%d0,-(%sp)
     move.l  #0x600001,%a0
     move.b  (%a0),%d0
     and.b   #0xfb,%d0
     move.b  %d0,(%a0)
     movem.l (%sp)+,%a0/%d0
     rts");

extern void EnableWriteProtection(void);
asm ("EnableWriteProtection:
     movem.l %a0/%d0,-(%sp)
     move.l  #0x600001,%a0
     move.b  (%a0),%d0
     or.b   #0x04,%d0
     move.b  %d0,(%a0)
     movem.l (%sp)+,%a0/%d0
     rts");
*/


//=============================================================================
// backups interrupts 1-7 and install special error vectors
// if you nesting this calls
//=============================================================================
void InstallCrashProtection(PROTECTION_T* backup) {
    short i = 16;

    OSSetSR(0x700); // interrupts off
    memcpy(&((*backup)[16]),(void*)0x40064, 7*4); // Saves the 7 auto_ints
    while (i--) {
        unsigned long **vector = (unsigned long **)(vec_offsets[i]+0x40000);
        (*backup)[i] = (unsigned long)*vector;
        *vector = (unsigned long *)(((unsigned char*)exceptionhandlers)+12*i);
    }

    RESET_EXCEPTION();

    OSSetSR(0); // interrupts on
}

//=============================================================================
// restores interrupts 1-7 and the error vectors
//=============================================================================
void RemoveCrashProtection(PROTECTION_T* backup) {
    short i = 16;

    OSSetSR(0x700);  // interrupts off
    memcpy((void*)0x40064, &((*backup)[16]), 7*4); // restore the 7 auto_ints
    while (i--) {
        unsigned long **vector = (unsigned long **)(vec_offsets[i]+0x40000);
        *vector = (unsigned long*)(*backup)[i];
    }
    OSSetSR(0); // interrupts on
}


//#############################################################################
// Revision History
//#############################################################################
//
// $Log: protect.c,v $
// Revision 1.3  2002/02/07 18:01:18  tnussb
// generic commit
//
// Revision 1.2  2001/01/26 21:04:44  Thomas Nussbaumer
// changes for version 0.80 [see history.txt]
//
// Revision 1.1  2001/01/20 21:33:37  Thomas Nussbaumer
// initial version
//
//
