/******************************************************************************
*
* project name:  TICT-Explorer
* file name:     viewhex.c
* initial date:  07/01/2001
* author:        thomas.nussbaumer@gmx.net
*
* description:   hexcode view routine
*
* $Id: viewers.c,v 1.6 2002/02/25 12:54:46 tnussb Exp $
*
******************************************************************************/

//=============================================================================
// integrated picture viewer
//=============================================================================
void ViewPIC(HANDLE handle) {
    unsigned char* src;
    unsigned short offx;
    unsigned short offy;

    if (!(src=SaveHeapLock(handle))) return;

    offy = (PIX_HEIGHT - ((BITMAP*)(src+2))->NumRows)/2;
    offx = (PIX_WIDTH - ((BITMAP*)(src+2))->NumCols)/2;

    memset(LCD_MEM,0,LCD_SIZE);
    BitmapPut(offx,offy,src+2,&fullscreen,A_REPLACE);
    GetUserInput();

    SaveHeapUnLock();
}


//=============================================================================
// put a single byte on the screen
//=============================================================================
void PutHexNumber(short x, short y, unsigned char value) {
    static const char hex[] = "0123456789ABCDEF";
    DrawChar(x,y,hex[(value >> 4) & 0x0f],A_REPLACE);
    DrawChar(x+6,y,hex[value & 0x0f],A_REPLACE);
}

#define HEX_NR_LINES 10
#define HEX_NR_BYTES 6
#define HEX_BYTES_PER_PAGE (HEX_NR_LINES*HEX_NR_BYTES)

//=============================================================================
// display a complete page
//=============================================================================
void DisplayPage(unsigned char* src,unsigned short idx,unsigned char* end) {
    short          x,y;
    unsigned char* act = src + idx;

    memset(LCD_MEM,0,LCD_SIZE);

    for (y=0;y<HEX_NR_LINES;y++) {
        PutHexNumber(0+OFFSET_X,y*10+OFFSET_Y,(unsigned char)((idx>>8)&0xff));
        PutHexNumber(12+OFFSET_X,y*10+OFFSET_Y,(unsigned char)(idx & 0xff));
        idx+=6;
        for (x=0;x<HEX_NR_BYTES;x++) {
            PutHexNumber(x*13+OFFSET_X+6*4+3,y*10+OFFSET_Y,*act);
            DrawChar(OFFSET_X+108+x*8,y*10+OFFSET_Y,*act,A_REPLACE);
            act++;
            if (act==end) return;
        }
    }
}

//=============================================================================
// integrated hexcode viewer
//=============================================================================
void ViewHEX(SYM_ENTRY* symptr) {
    unsigned char* src;
    unsigned char* end;
    unsigned short size;
    unsigned short input;
    long           index  = 0;
    int            redraw = 1;

    if (!(src=SaveHeapLock(symptr->handle))) return;

    size = *(unsigned short*)(src) + 2;

    end = src+size;

    FontSetSys(F_6x8);

    do {
        if (redraw) {
            DisplayPage(src,(unsigned short)index,end);
            redraw = 0;
        }
        input = GetUserInput();
        if (input == KEY_UP) {
            if (index >= HEX_BYTES_PER_PAGE) {
                index -= HEX_BYTES_PER_PAGE;
                redraw = 1;
            }
            else {
                index = size / HEX_BYTES_PER_PAGE;
                index *= HEX_BYTES_PER_PAGE;
                redraw = 1;
            }
        }
        else if (input == KEY_DOWN) {
            if (index+HEX_BYTES_PER_PAGE <= size) {
                index += HEX_BYTES_PER_PAGE;
                redraw = 1;
            }
            else {
                index  = 0;
                redraw = 1;
            }
        }
        else if (input == KEY_UP2ND) {   // 2nd + UP
            index  = 0;
            redraw = 1;
        }
        else if (input == KEY_DOWN2ND) { // 2nd + DOWN
            index = size / HEX_BYTES_PER_PAGE;
            index *= HEX_BYTES_PER_PAGE;
            redraw = 1;
        }
    }
    while (input != KEY_ESC);

    FontSetSys(F_4x6);

    SaveHeapUnLock();
}


//=============================================================================
// integrated text viewer
//=============================================================================
#define ASCII_PER_PAGE (20*HEX_NR_LINES)

unsigned char* StartOfPage(unsigned short pagenr,unsigned char* start,unsigned char* end) {
    unsigned short line_count = 0;
    unsigned char* tmp        = start;
    unsigned short char_count = 0;
    unsigned short page_count = 0;

    while (page_count != pagenr && tmp <= end) {
        if (*tmp == 0x0d) {
            line_count++;
            char_count=0;
            tmp++;
            if (tmp>end) break;
        }
        else {
            char_count++;
            if (char_count==20) {
                line_count++;
                char_count=0;
            }
        }

        if (line_count == 10) {
            page_count++;
            line_count=0;
        }
        tmp++;
    }

    if (tmp > end) return NULL;

    return tmp;
}



void ViewTEXT(SYM_ENTRY* symptr) {
    unsigned char* src;
    unsigned char* end;
    unsigned short size;
    unsigned short input;
    short          redraw = 1;
    unsigned short pages;
    long           active_page = 0;

    if (!(src=SaveHeapLock(symptr->handle))) return;

    size = *(unsigned short*)(src) + 2;

    src+=5;
    size-=7;

    end = src+size;

    for (pages = 0;pages<65520/20;pages++) {
        if (!StartOfPage(pages,src,end)) break;
    }

    FontSetSys(F_6x8);

    do {
        if (redraw) {
            unsigned char* act = StartOfPage((unsigned short)active_page,src,end);
            short x,y;
            unsigned char c;

            memset(LCD_MEM,0,LCD_SIZE);

            for (y=0;y<HEX_NR_LINES;y++) {
                for (x=0;x<20;x++) {
                    c = *act++;
                    if (c==0xd) {
                        act++;
                        break;
                    }
                    if (c) DrawChar(OFFSET_X+x*8,y*10+OFFSET_Y,c,A_REPLACE);
                    if (act>=end) break;
                }
                if (act>=end) break;
            }
            redraw = 0;
        }

        input = GetUserInput();
        if (input == KEY_UP) {
            active_page--;
            if (active_page < 0) active_page = pages-1;
            redraw = 1;
        }
        else if (input == KEY_DOWN) {
            active_page++;
            if (active_page >= pages) active_page = 0;
            redraw = 1;
        }
        else if (input == KEY_UP2ND) {   // 2nd + UP
            active_page = 0;
            redraw = 1;
        }
        else if (input == KEY_DOWN2ND) { // 2nd + DOWN
            active_page = pages-1;
            redraw = 1;
        }
    }
    while (input != KEY_ESC);

    FontSetSys(F_4x6);

    SaveHeapUnLock();
    DrawStatusLine(NULL);
}

//=============================================================================
// Revision History
//=============================================================================
//
// $Log: viewers.c,v $
// Revision 1.6  2002/02/25 12:54:46  tnussb
// minor size optimizations and beautifying
//
// Revision 1.5  2002/02/07 18:01:19  tnussb
// generic commit
//
// Revision 1.4  2001/02/04 13:11:28  Thomas Nussbaumer
// changes up to version 1.00 RC2 (see history.txt)
//
// Revision 1.3  2001/01/26 21:04:44  Thomas Nussbaumer
// changes for version 0.80 [see history.txt]
//
//
